"""
    Cisco Intersight

    Cisco Intersight is a management platform delivered as a service with embedded analytics for your Cisco and 3rd party IT infrastructure. This platform offers an intelligent level of management that enables IT organizations to analyze, simplify, and automate their environments in more advanced ways than the prior generations of tools. Cisco Intersight provides an integrated and intuitive management experience for resources in the traditional data center as well as at the edge. With flexible deployment options to address complex security needs, getting started with Intersight is quick and easy. Cisco Intersight has deep integration with Cisco UCS and HyperFlex systems allowing for remote deployment, configuration, and ongoing maintenance. The model-based deployment works for a single system in a remote location or hundreds of systems in a data center and enables rapid, standardized configuration and deployment. It also streamlines maintaining those systems whether you are working with small or very large configurations. The Intersight OpenAPI document defines the complete set of properties that are returned in the HTTP response. From that perspective, a client can expect that no additional properties are returned, unless these properties are explicitly defined in the OpenAPI document. However, when a client uses an older version of the Intersight OpenAPI document, the server may send additional properties because the software is more recent than the client. In that case, the client may receive properties that it does not know about. Some generated SDKs perform a strict validation of the HTTP response body against the OpenAPI document.  # noqa: E501

    The version of the OpenAPI document: 1.0.11-20241004054146473
    Contact: intersight@cisco.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from intersight.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    set_model_init_error,
    OpenApiModel
)
from intersight.exceptions import ApiAttributeError


def lazy_import():
    from intersight.model.mo_base_mo_relationship import MoBaseMoRelationship
    from intersight.model.vnic_fc_error_recovery_settings import VnicFcErrorRecoverySettings
    from intersight.model.vnic_fc_interrupt_settings import VnicFcInterruptSettings
    from intersight.model.vnic_fc_queue_settings import VnicFcQueueSettings
    from intersight.model.vnic_flogi_settings import VnicFlogiSettings
    from intersight.model.vnic_plogi_settings import VnicPlogiSettings
    from intersight.model.vnic_scsi_queue_settings import VnicScsiQueueSettings
    globals()['MoBaseMoRelationship'] = MoBaseMoRelationship
    globals()['VnicFcErrorRecoverySettings'] = VnicFcErrorRecoverySettings
    globals()['VnicFcInterruptSettings'] = VnicFcInterruptSettings
    globals()['VnicFcQueueSettings'] = VnicFcQueueSettings
    globals()['VnicFlogiSettings'] = VnicFlogiSettings
    globals()['VnicPlogiSettings'] = VnicPlogiSettings
    globals()['VnicScsiQueueSettings'] = VnicScsiQueueSettings


class VnicFcAdapterPolicyInventoryAllOf(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('class_id',): {
            'VNIC.FCADAPTERPOLICYINVENTORY': "vnic.FcAdapterPolicyInventory",
        },
        ('object_type',): {
            'VNIC.FCADAPTERPOLICYINVENTORY': "vnic.FcAdapterPolicyInventory",
        },
    }

    validations = {
        ('error_detection_timeout',): {
            'inclusive_maximum': 100000,
            'inclusive_minimum': 1000,
        },
        ('io_throttle_count',): {
            'inclusive_maximum': 1024,
            'inclusive_minimum': 1,
        },
        ('lun_count',): {
            'inclusive_maximum': 4096,
            'inclusive_minimum': 1,
        },
        ('lun_queue_depth',): {
            'inclusive_maximum': 254,
            'inclusive_minimum': 1,
        },
        ('resource_allocation_timeout',): {
            'inclusive_maximum': 100000,
            'inclusive_minimum': 5000,
        },
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'class_id': (str,),  # noqa: E501
            'object_type': (str,),  # noqa: E501
            'error_detection_timeout': (int,),  # noqa: E501
            'error_recovery_settings': (VnicFcErrorRecoverySettings,),  # noqa: E501
            'flogi_settings': (VnicFlogiSettings,),  # noqa: E501
            'interrupt_settings': (VnicFcInterruptSettings,),  # noqa: E501
            'io_throttle_count': (int,),  # noqa: E501
            'lun_count': (int,),  # noqa: E501
            'lun_queue_depth': (int,),  # noqa: E501
            'plogi_settings': (VnicPlogiSettings,),  # noqa: E501
            'resource_allocation_timeout': (int,),  # noqa: E501
            'rx_queue_settings': (VnicFcQueueSettings,),  # noqa: E501
            'scsi_queue_settings': (VnicScsiQueueSettings,),  # noqa: E501
            'tx_queue_settings': (VnicFcQueueSettings,),  # noqa: E501
            'target_mo': (MoBaseMoRelationship,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'class_id': 'ClassId',  # noqa: E501
        'object_type': 'ObjectType',  # noqa: E501
        'error_detection_timeout': 'ErrorDetectionTimeout',  # noqa: E501
        'error_recovery_settings': 'ErrorRecoverySettings',  # noqa: E501
        'flogi_settings': 'FlogiSettings',  # noqa: E501
        'interrupt_settings': 'InterruptSettings',  # noqa: E501
        'io_throttle_count': 'IoThrottleCount',  # noqa: E501
        'lun_count': 'LunCount',  # noqa: E501
        'lun_queue_depth': 'LunQueueDepth',  # noqa: E501
        'plogi_settings': 'PlogiSettings',  # noqa: E501
        'resource_allocation_timeout': 'ResourceAllocationTimeout',  # noqa: E501
        'rx_queue_settings': 'RxQueueSettings',  # noqa: E501
        'scsi_queue_settings': 'ScsiQueueSettings',  # noqa: E501
        'tx_queue_settings': 'TxQueueSettings',  # noqa: E501
        'target_mo': 'TargetMo',  # noqa: E501
    }

    read_only_vars = {
        'error_detection_timeout',  # noqa: E501
        'io_throttle_count',  # noqa: E501
        'lun_count',  # noqa: E501
        'lun_queue_depth',  # noqa: E501
        'resource_allocation_timeout',  # noqa: E501
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """VnicFcAdapterPolicyInventoryAllOf - a model defined in OpenAPI

        Args:

        Keyword Args:
            class_id (str): The fully-qualified name of the instantiated, concrete type. This property is used as a discriminator to identify the type of the payload when marshaling and unmarshaling data.. defaults to "vnic.FcAdapterPolicyInventory", must be one of ["vnic.FcAdapterPolicyInventory", ]  # noqa: E501
            object_type (str): The fully-qualified name of the instantiated, concrete type. The value should be the same as the 'ClassId' property.. defaults to "vnic.FcAdapterPolicyInventory", must be one of ["vnic.FcAdapterPolicyInventory", ]  # noqa: E501
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            error_detection_timeout (int): Error Detection Timeout, also referred to as EDTOV, is the number of milliseconds to wait before the system assumes that an error has occurred.. [optional] if omitted the server will use the default value of 2000  # noqa: E501
            error_recovery_settings (VnicFcErrorRecoverySettings): [optional]  # noqa: E501
            flogi_settings (VnicFlogiSettings): [optional]  # noqa: E501
            interrupt_settings (VnicFcInterruptSettings): [optional]  # noqa: E501
            io_throttle_count (int): The maximum number of data or control I/O operations that can be pending for the virtual interface at one time. If this value is exceeded, the additional I/O operations wait in the queue until the number of pending I/O operations decreases and the additional operations can be processed.. [optional] if omitted the server will use the default value of 512  # noqa: E501
            lun_count (int): The maximum number of LUNs that the Fibre Channel driver will export or show. The maximum number of LUNs is usually controlled by the operating system running on the server. Lun Count value can exceed 1024 only for vHBA of type 'FC Initiator' and on servers having supported firmware version.. [optional] if omitted the server will use the default value of 1024  # noqa: E501
            lun_queue_depth (int): The number of commands that the HBA can send and receive in a single transmission per LUN.. [optional] if omitted the server will use the default value of 20  # noqa: E501
            plogi_settings (VnicPlogiSettings): [optional]  # noqa: E501
            resource_allocation_timeout (int): Resource Allocation Timeout, also referred to as RATOV, is the number of milliseconds to wait before the system assumes that a resource cannot be properly allocated.. [optional] if omitted the server will use the default value of 10000  # noqa: E501
            rx_queue_settings (VnicFcQueueSettings): [optional]  # noqa: E501
            scsi_queue_settings (VnicScsiQueueSettings): [optional]  # noqa: E501
            tx_queue_settings (VnicFcQueueSettings): [optional]  # noqa: E501
            target_mo (MoBaseMoRelationship): [optional]  # noqa: E501
        """

        class_id = "vnic.FcAdapterPolicyInventory" if len(kwargs.get('class_id','').strip()) == 0 else kwargs.get('class_id', "vnic.FcAdapterPolicyInventory")
        object_type = "vnic.FcAdapterPolicyInventory" if len(kwargs.get('object_type','').strip()) == 0 else kwargs.get('object_type', "vnic.FcAdapterPolicyInventory")
        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', True)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.class_id = class_id
        self.object_type = object_type
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """VnicFcAdapterPolicyInventoryAllOf - a model defined in OpenAPI

        Args:

        Keyword Args:
            class_id (str): The fully-qualified name of the instantiated, concrete type. This property is used as a discriminator to identify the type of the payload when marshaling and unmarshaling data.. defaults to "vnic.FcAdapterPolicyInventory", must be one of ["vnic.FcAdapterPolicyInventory", ]  # noqa: E501
            object_type (str): The fully-qualified name of the instantiated, concrete type. The value should be the same as the 'ClassId' property.. defaults to "vnic.FcAdapterPolicyInventory", must be one of ["vnic.FcAdapterPolicyInventory", ]  # noqa: E501
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            error_detection_timeout (int): Error Detection Timeout, also referred to as EDTOV, is the number of milliseconds to wait before the system assumes that an error has occurred.. [optional] if omitted the server will use the default value of 2000  # noqa: E501
            error_recovery_settings (VnicFcErrorRecoverySettings): [optional]  # noqa: E501
            flogi_settings (VnicFlogiSettings): [optional]  # noqa: E501
            interrupt_settings (VnicFcInterruptSettings): [optional]  # noqa: E501
            io_throttle_count (int): The maximum number of data or control I/O operations that can be pending for the virtual interface at one time. If this value is exceeded, the additional I/O operations wait in the queue until the number of pending I/O operations decreases and the additional operations can be processed.. [optional] if omitted the server will use the default value of 512  # noqa: E501
            lun_count (int): The maximum number of LUNs that the Fibre Channel driver will export or show. The maximum number of LUNs is usually controlled by the operating system running on the server. Lun Count value can exceed 1024 only for vHBA of type 'FC Initiator' and on servers having supported firmware version.. [optional] if omitted the server will use the default value of 1024  # noqa: E501
            lun_queue_depth (int): The number of commands that the HBA can send and receive in a single transmission per LUN.. [optional] if omitted the server will use the default value of 20  # noqa: E501
            plogi_settings (VnicPlogiSettings): [optional]  # noqa: E501
            resource_allocation_timeout (int): Resource Allocation Timeout, also referred to as RATOV, is the number of milliseconds to wait before the system assumes that a resource cannot be properly allocated.. [optional] if omitted the server will use the default value of 10000  # noqa: E501
            rx_queue_settings (VnicFcQueueSettings): [optional]  # noqa: E501
            scsi_queue_settings (VnicScsiQueueSettings): [optional]  # noqa: E501
            tx_queue_settings (VnicFcQueueSettings): [optional]  # noqa: E501
            target_mo (MoBaseMoRelationship): [optional]  # noqa: E501
        """

        class_id = "vnic.FcAdapterPolicyInventory" if len(kwargs.get('class_id','').strip()) == 0 else kwargs.get('class_id', "vnic.FcAdapterPolicyInventory")
        object_type = "vnic.FcAdapterPolicyInventory" if len(kwargs.get('object_type','').strip()) == 0 else kwargs.get('object_type', "vnic.FcAdapterPolicyInventory")
        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.class_id = class_id
        self.object_type = object_type
        for var_name, var_value in kwargs.items():
            if var_name in self.read_only_vars:
                set_model_init_error(f"Warning : {var_name} is omitted from request payload as it is a read-only property")
                continue
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
